module AudioFeedManager
  class Application
    include Dependor::AutoInject
    look_in_modules AudioFeedManager

    attr_reader :directory, :stdin, :stderr, :stdout

    def initialize(directory:, stdin:, stderr:, stdout:)
      @directory = Pathname.new(File.expand_path(directory))
      @stdin = stdin
      @stderr = stderr
      @stdout = stdout
    end

    def self.let(name, &block)
      define_method(name) do
        @object_cache ||= {}
        @object_cache[name] ||= instance_exec(&block)
      end
    end

    let(:cli) { inject(CLI) }
    let(:help) { commands_list }
    let(:feeds_prefix) { "feeds" }
    let(:feeds_directory) { directory.join(feeds_prefix) }
    let(:audio_files_prefix) { "audio_files" }
    let(:audio_files_directory) { directory.join(audio_files_prefix) }
    let(:rss_files_prefix) { "rss" }
    let(:rss_files_directory) { directory.join(rss_files_prefix) }
    let(:data_files_prefix) { "data" }
    let(:data_files_directory) { directory.join(data_files_prefix) }
    let(:items_prefix) { "items" }
    let(:items_directory) { directory.join(items_prefix) }
    let(:storage) { file_storage }
    let(:rss_repository) { inject(RSSRepository) }
    let(:config_file) { directory.join("config.yml") }
    let(:config) { config_repository.load(config_file) }
  end
end
