require 'fileutils'

module AudioFeedManager
  class AudioFileRepository
    takes :new_model_creator, :audio_files_directory, :audio_files_prefix, :data_files_directory, :storage

    def add(audio_file, file_name)
      add_file_information(audio_file, file_name)
      audio_file = new_model_creator.create(audio_file,
                                            directory: audio_files_directory,
                                            prefix: audio_files_prefix)
      copy_file(file_name, audio_file.file_name)

      audio_file
    end

    def fetch(id)
      attrs = storage.read(id)
      AudioFile.new(attrs)
    rescue FileNotFound
      raise AudioFileNotFound.for_id(id)
    end

    private

    def add_file_information(audio_file, file_name)
      audio_file.extension = File.extname(file_name)
      audio_file.size = File.size(file_name)
      audio_file.content_type = `file --mime-type --brief #{file_name}`.chomp
    end

    def copy_file(file, new_name)
      FileUtils.cp(file, data_files_directory.join(new_name))
    end
  end

  class AudioFileNotFound < Error
    def self.for_id(id)
      new("AudioFile by id: #{id} was not found!")
    end
  end
end
