module AudioFeedManager
  class AddAudioFile < Command
    arguments do |s|
      s.name "audio add"
      s.description "Add a mp3 file to the specified feed"

      s.argument :feed, required: true, description: "ID of the feed to add the file to, ex. feeds/hello-world"
      s.varargs :files, minimum: 1, description: "Files to add"
    end

    takes :feed_repository, :audio_file_adder, :rss_generator, :console, :directory

    def run(feed:, files:)
      feed = feed_repository.fetch(feed)

      files.each do |file|
        add_file(feed, file)
      end

      rss_generator.generate(feed)
    rescue FeedNotFound => e
      console.die(e)
    rescue SystemCallError => e
      console.die(e)
    end

    private

    def add_file(feed, file)
      file_name = Pathname.new(directory.join(file))

      console.info("Adding #{file_name} to #{feed.id}")

      audio_file_adder.call feed, file_name,
        title: ->() { console.ask("No title tag found. Please provide title", default: File.basename(file_name)) },
        author: ->() { console.ask("No artist tag found. Please provide author") }
    end
  end
end
