module AudioFeedManager
  class ArgumentsParser
    def parse(specification, arguments)
      result = Arguments.new

      result, arguments = parse_required_arguments(result, arguments, specification)
      result, arguments = parse_optional_arguments(result, arguments, specification)
      result, arguments = parse_varargs(result, arguments, specification)

      if arguments.any?
        raise InvalidArguments, "Provided #{arguments.size} extra arguments!"
      end

      result
    end

    private

    def parse_required_arguments(result, arguments, specification)
      if arguments.size < specification.required_arguments.size
        raise InvalidArguments,
          "Provided #{arguments.size} arguments, but #{specification.required_arguments.size} are required."
      end

      extract_arguments(result, arguments, specification.required_arguments)
    end

    def parse_varargs(result, arguments, specification)
      return [result, arguments] unless specification.varargs_arguments.any?

      if arguments.size < specification.minimum_varags_count
        raise InvalidArguments,
          "Provided #{arguments.size} for variable-length arguments, but #{specification.minimum_varags_count} are required."
      end

      specs = specification.varargs_arguments
      arguments = arguments.dup

      specs[0..-2].each do |arg|
        value = arguments.shift(arg.minimum)
        result.add_argument(arg.name, value)
      end

      result.add_argument(specs[-1].name, arguments)

      [result, []]
    end

    def parse_optional_arguments(result, arguments, specification)
      extract_arguments(result, arguments, specification.optional_arguments)
    end

    def extract_arguments(result, arguments, specs)
      arguments = arguments.dup

      specs.each do |arg|
        value = arguments.shift
        result.add_argument(arg.name, value)
      end

      [result, arguments]
    end
  end

  class InvalidArguments < Error
  end
end
