module AudioFeedManager
  class ArgumentsSpecification
    extend GetterSetterMethod

    Argument = Struct.new(:name, :required, :description, :default) do
      def usage_name
        name.upcase
      end

      def optional
        !required
      end

      def varargs
        false
      end
    end

    Varargs = Struct.new(:name, :minimum, :description) do
      def usage_name
        "#{name.upcase}..."
      end

      def required
        false
      end

      def optional
        false
      end

      def varargs
        true
      end
    end

    def self.no_arguments
      new
    end

    def self.specify
      specification = no_arguments
      yield specification
      specification
    end

    get_set(:name) { raise "Command name not specified" }
    get_set(:description) { raise "Description not specified" }

    def argument(name, required: false, description: "", default: nil)
      arguments << Argument.new(name, required, description, default)
    end

    def varargs(name, minimum: 1, description: "")
      arguments << Varargs.new(name, minimum, description)
    end

    def minimum_varags_count
      varargs_arguments.map(&:minimum).reduce(0, :+)
    end

    def required_arguments
      arguments.select(&:required)
    end

    def optional_arguments
      arguments.select(&:optional)
    end

    def varargs_arguments
      arguments.select(&:varargs)
    end

    def arguments
      @arguments ||= []
    end

    def usage_line
      name + required_arguments_line + optional_arguments_line + varargs_line
    end

    def has_arguments?
      !arguments.empty?
    end

    private

    def required_arguments_line
      return "" if required_arguments.empty?
      " " + required_arguments.map(&:usage_name).map{|s| "<#{s}>"}.join(" ")
    end

    def optional_arguments_line
      return "" if optional_arguments.empty?
      " " + optional_arguments.map(&:usage_name).map{|s| "[#{s}]"}.join(" ")
    end

    def varargs_line
      return "" if varargs_arguments.empty?
      " " + varargs_arguments.map(&:usage_name).map{|s| "[#{s}]"}.join(" ")
    end
  end
end
