require "fileutils"

module AudioFeedManager
  class InitializeProject < Command
    takes :directory,
      :console,
      :config_repository,
      :rss_files_prefix,
      :audio_files_prefix,
      :data_files_prefix,
      :items_prefix,
      :feeds_prefix

    arguments do |s|
      s.name 'init'
      s.description 'Initialize a new project directory'

      s.argument :project_dir, required: true, description: "Name of the project (will be used to create a directory)"
    end

    def run(project_dir:)
      project_dir = directory.join(project_dir)
      mkdir(project_dir, rss_files_prefix)
      mkdir(project_dir, audio_files_prefix)
      mkdir(project_dir, data_files_prefix)
      mkdir(project_dir, items_prefix)
      mkdir(project_dir, feeds_prefix)

      config = Config.new(
        s3_access_key: console.ask("S3 Access key"),
        s3_secret: console.ask("S3 Secret key"),
        s3_region: console.ask("S3 Region", default: "us-west-2"),
        s3_bucket: console.ask("Bucket name")
      )
      config_repository.store(project_dir.join("config.yml"), config)
    end

    private

    def mkdir(project_dir, prefix)
      FileUtils.mkdir_p(project_dir.join(prefix))
    end
  end
end
