module AudioFeedManager
  class ShowFeed < Command
    arguments do |s|
      s.name 'feeds show'
      s.description 'Show detailed information about the feed'

      s.argument :feed, required: true, description: "Feed to show information about, ex. feeds/hello-world"
    end

    takes :feed_repository, :feed_items_repository, :url_maker, :console

    def run(feed:)
      feed = feed_repository.fetch(feed)
      audio_files = feed_items_repository.list(feed)

      print_details(feed)
      separator
      print_audio_files(audio_files)
    rescue FeedNotFound => e
      console.die(e)
    end

    private

    def print_details(feed)
      console.info "Title: #{feed.title}"
      console.info "ID: #{feed.id}"
      console.info "URL: #{url_maker.feed_url(feed)}"
    end

    def separator
      console.info ""
    end

    def print_audio_files(audio_files)
      return console.info "No audio files. Add one with 'afm audio add <FILE>'." if audio_files.empty?

      console.info("#\tID\tTitle")
      audio_files.each_with_index do |audio_file, idx|
        console.info "#{idx + 1}\t#{audio_file.id}\t#{audio_file.title}"
      end
    end
  end
end
