require 'aws-sdk'
require 'digest'

module AudioFeedManager
  class S3Gateway
    takes :config

    def url(path)
      bucket.object(path).public_url
    end

    def sync(key, file_name, &block)
      return unless needs_syncing?(key, file_name)
      yield key if block_given?

      object = bucket.object(key)
      object.upload_file(file_name)
      object.acl.put(acl: "public-read")
    end

    private

    def needs_syncing?(key, file_name)
      object = bucket.object(key)
      return true unless object.exists?
      object.etag.gsub('"', '') != Digest::MD5.file(file_name).to_s
    end

    def s3
      @s3 ||= Aws::S3::Resource.new(access_key_id: config.s3_access_key,
                                    secret_access_key: config.s3_secret,
                                    region: config.s3_region)
    end

    def bucket
      @bucket ||= s3.bucket(config.s3_bucket)
    end
  end
end
